#define WIN32_LEAN_AND_MEAN
#define WINDOWX	640
#define WINDOWY	480

#define STATE_INIT	0
#define STATE_MAIN	1
#define STATE_QUIT	2

#define SizeX 640
#define SizeY 480
#define BPP 32

#define BN 10

#include <windows.h>
#include <math.h>
#include "nucgfx3.h"
#include "3dmath.h"
#include "3dclass.h"

void Project(Vertex *vertex);
void DrawWireQuad(Quad *quads, int QuadNumber, BYTE *buffer, long pitch);
void DrawWireTriangle(Triangle *triang, BYTE *buffer, long pitch);

LRESULT CALLBACK WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

void Init();
void Main();
void CleanUp();

HWND hWnd;
int State = STATE_INIT;

const int MAX_X = 640;
const int MAX_Y = 480;
const float pi = 3.1415926535f;
const float quarter_pi = pi/4.0f;
const float rotrads = 0.01f;
float yangle;
float xangle;


int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR CmdLine, int ShowCmd) {

	MSG msg;
	WNDCLASSEX wc;
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_VREDRAW | CS_HREDRAW;
	wc.lpfnWndProc = WinProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInst;
	wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wc.hIconSm = wc.hIcon;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = "NucWin";
	RegisterClassEx(&wc);

	hWnd = CreateWindowEx(	NULL,
							"NucWin",
							"Nina/ASD",
							NUCGFX_WINDOWSTYLE_WINDOWED,
							(GetSystemMetrics(SM_CXSCREEN)-WINDOWX)/2,
							(GetSystemMetrics(SM_CYSCREEN)-WINDOWY)/2,
							WINDOWX, WINDOWY,
							NULL,
							NULL,
							hInst,
							NULL );
	ShowWindow(hWnd, ShowCmd);
	UpdateWindow(hWnd);

	Init();
	State = STATE_MAIN;

	while(1) {
   		if(PeekMessage(&msg, NULL, 0,0, PM_REMOVE)) {
      		if(msg.message == WM_QUIT) break;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		} 
		else  {
      	// main loop
			if(State == STATE_MAIN) Main();
		}
	}
	return msg.wParam;
}

LRESULT CALLBACK WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {

	switch(msg) {
	case WM_KEYDOWN:
   	if(wParam == VK_ESCAPE) {
      	PostMessage(hWnd, WM_CLOSE, 0, 0);
         return 0;
	}
	if(wParam == VK_LEFT) {
		yangle -= rotrads;
		return 0;
	}
	if(wParam == VK_RIGHT) {
		yangle += rotrads;
		return 0;
	}
	if(wParam == VK_UP) {
		xangle -= rotrads;
		return 0;
	}
	if(wParam == VK_DOWN) {
		xangle += rotrads;
		return 0;
	}

	break;
//	case WM_MOVE:
//    if(gfx) gfx->WindowMoved();
//    return 0;
//	break;
	case WM_CLOSE:
   		State = STATE_QUIT;
   		CleanUp();
   		DestroyWindow(hWnd);
		return 0;
	break;

	case WM_DESTROY:
   		PostQuitMessage(0);
	break;

	default: break;
	}

	return DefWindowProc(hWnd, msg, wParam, lParam);
}

NucGFX *gfx;
NucGFXSurface *surf;
Vertex vert[8];				// cube vertices
Vertex vert2[8];			// cube 2 vertices
Vertex tvert[3];			// pyramid vertices
Matrix4x4 wcstm;			// world coordinate system transformation matrix
Matrix4x4 wcstm2;			// world coordinate system transformation matrix
Matrix4x4 wcstrans;			// world coordinate system translation matrix
Matrix4x4 wcstrans2;			// world coordinate system translation matrix
Quad cube[6];
Quad cube2[6];
Triangle trian;
Vertex pervert[8];				// salooooooooooooooooooooooo
Vertex pervert2[8];				// salooooooooooooooooooooooo
Vertex tpervert[3];
float FOV = quarter_pi;
float dist = (float)WINDOWX/(float)(2*tan(FOV/2));

void Init() {
	gfx = new NucGFX(SizeX, SizeY, BPP, NUCGFX_WINDOWED, 2);

	ShowCursor(FALSE);

	////////////  cube  //////////////

	vert[0] = Vertex(1,1,-1);			// local coordinates
	vert[1] = Vertex(1,-1,-1);
	vert[2] = Vertex(-1,-1,-1);
	vert[3] = Vertex(-1,1,-1);
	vert[4] = Vertex(1,1,1);
	vert[5] = Vertex(1,-1,1);
	vert[6] = Vertex(-1,-1,1);
	vert[7] = Vertex(-1,1,1);

	vert2[0] = Vertex(1,1,-1);			// local coordinates
	vert2[1] = Vertex(1,-1,-1);
	vert2[2] = Vertex(-1,-1,-1);
	vert2[3] = Vertex(-1,1,-1);
	vert2[4] = Vertex(1,1,1);
	vert2[5] = Vertex(1,-1,1);
	vert2[6] = Vertex(-1,-1,1);
	vert2[7] = Vertex(-1,1,1);

	for(int i=0; i<8; i++)
		vert[i].SetColor(0, 0, 255);

	for(i=0; i<8; i++)
		vert2[i].SetColor(0, 0, 255);

	cube[0] = Quad(&pervert[0], &pervert[1], &pervert[2], &pervert[3]);
	cube[1] = Quad(&pervert[3], &pervert[2], &pervert[6], &pervert[7]);
	cube[2] = Quad(&pervert[4], &pervert[7], &pervert[6], &pervert[5]);
	cube[3] = Quad(&pervert[0], &pervert[4], &pervert[5], &pervert[1]);
	cube[4] = Quad(&pervert[7], &pervert[4], &pervert[0], &pervert[3]);
	cube[5] = Quad(&pervert[5], &pervert[6], &pervert[2], &pervert[1]);

	cube2[0] = Quad(&pervert2[0], &pervert2[1], &pervert2[2], &pervert2[3]);
	cube2[1] = Quad(&pervert2[3], &pervert2[2], &pervert2[6], &pervert2[7]);
	cube2[2] = Quad(&pervert2[4], &pervert2[7], &pervert2[6], &pervert2[5]);
	cube2[3] = Quad(&pervert2[0], &pervert2[4], &pervert2[5], &pervert2[1]);
	cube2[4] = Quad(&pervert2[7], &pervert2[4], &pervert2[0], &pervert2[3]);
	cube2[5] = Quad(&pervert2[5], &pervert2[6], &pervert2[2], &pervert2[1]);

	tvert[0] = Vertex(0,1,0);
	tvert[1] = Vertex(-1,0,0);
	tvert[2] = Vertex(1,0,0);

	for(i=0; i<3; i++)
		tvert[i].SetColor(0, 0, 255);

	trian = Triangle(&tpervert[0], &tpervert[1], &tpervert[2]);	
	
	wcstrans.Translate(0, 0, 10);
	wcstrans2.Translate((float)0.05, 0, 10);
}

void CleanUp() 
{
	delete gfx;
	ShowCursor(TRUE);
}

float angle = 0;

void Main() 
{
	unsigned int i;
	Matrix4x4 rot;
	rot.Rotate(xangle, yangle, 0);

	wcstm.reset();
	wcstm2.reset();
	wcstm *= rot;	
	wcstm2 *= rot;	
	wcstm *= wcstrans;
	wcstm2 *= wcstrans2;
		
	memcpy(pervert, vert, 8*sizeof(Vertex));
	memcpy(pervert2, vert2, 8*sizeof(Vertex));
	memcpy(tpervert, tvert, 3*sizeof(Vertex));
	
	for(i=0; i<8; i++)  {
		pervert[i].pos.transform(wcstm);
		Project(&pervert[i]);
	}

	for(i=0; i<8; i++)  {
		pervert2[i].pos.transform(wcstm2);
		Project(&pervert2[i]);
	}

	for(i=0; i<3; i++)  {
		tpervert[i].pos.transform(wcstm);
		Project(&tpervert[i]);
	}

	gfx->ClearSurf(0,0,0,gfx->BackBuffer);
	gfx->Lock(gfx->BackBuffer);

    // main2

	for(i=0; i<6; i++)  {
		DrawWireQuad(cube, 6, gfx->FrameBuffer, gfx->Pitch);
		DrawWireQuad(cube2, 6, gfx->FrameBuffer, gfx->Pitch);
	}

	DrawWireTriangle(&trian, gfx->FrameBuffer, gfx->Pitch);

	gfx->UnLock(gfx->BackBuffer);
	gfx->Flip();
	
//	angle += 0.000005f*(float)(GetTickCount() / 1000);

}

void DrawWireQuad(Quad *quads, int QuadNumber, BYTE *buffer, long pitch)
{
	for(int i=0; i<QuadNumber; i++)  {
		gfx->Line((WORD)quads[i].vert[0]->pos.x, (WORD)quads[i].vert[0]->pos.y, (WORD)quads[i].vert[1]->pos.x, (WORD)quads[i].vert[1]->pos.y, quads[i].vert[0]->r, quads[i].vert[0]->g, quads[i].vert[0]->b, buffer, pitch);
		gfx->Line((WORD)quads[i].vert[1]->pos.x, (WORD)quads[i].vert[1]->pos.y, (WORD)quads[i].vert[2]->pos.x, (WORD)quads[i].vert[2]->pos.y, quads[i].vert[1]->r, quads[i].vert[1]->g, quads[i].vert[1]->b, buffer, pitch);
		gfx->Line((WORD)quads[i].vert[2]->pos.x, (WORD)quads[i].vert[2]->pos.y, (WORD)quads[i].vert[3]->pos.x, (WORD)quads[i].vert[3]->pos.y, quads[i].vert[2]->r, quads[i].vert[2]->g, quads[i].vert[2]->b, buffer, pitch);
		gfx->Line((WORD)quads[i].vert[3]->pos.x, (WORD)quads[i].vert[3]->pos.y, (WORD)quads[i].vert[0]->pos.x, (WORD)quads[i].vert[0]->pos.y, quads[i].vert[3]->r, quads[i].vert[3]->g, quads[i].vert[3]->b, buffer, pitch);
	}
}

void DrawWireTriangle(Triangle *triang, BYTE *buffer, long pitch)
{
	gfx->Line((WORD)triang->vert[0]->pos.x, (WORD)triang->vert[0]->pos.y, (WORD)triang->vert[1]->pos.x, (WORD)triang->vert[1]->pos.y, triang->vert[0]->r, triang->vert[0]->g, triang->vert[0]->b, buffer, pitch);
	gfx->Line((WORD)triang->vert[1]->pos.x, (WORD)triang->vert[1]->pos.y, (WORD)triang->vert[2]->pos.x, (WORD)triang->vert[2]->pos.y, triang->vert[1]->r, triang->vert[1]->g, triang->vert[1]->b, buffer, pitch);
	gfx->Line((WORD)triang->vert[2]->pos.x, (WORD)triang->vert[2]->pos.y, (WORD)triang->vert[0]->pos.x, (WORD)triang->vert[0]->pos.y, triang->vert[2]->r, triang->vert[2]->g, triang->vert[2]->b, buffer, pitch);

}

void Project(Vertex *vertex) 
{
    vertex->pos.x = (dist*vertex->pos.x / vertex->pos.z) + (WINDOWX >> 1);
    vertex->pos.y = (WINDOWY >> 1) - (dist*vertex->pos.y / vertex->pos.z);
}
