#define WIN32_LEAN_AND_MEAN
#define WINDOWX	400
#define WINDOWY	300

#define STATE_INIT	0
#define STATE_MAIN	1
#define STATE_QUIT	2

#define SizeX 640
#define SizeY 480
#define BPP 32

#include <windows.h>
#include <math.h>
#include "nucgfx3.h"

LRESULT CALLBACK WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

void Init();
void Main();
void CleanUp();

HWND hWnd;
int State = STATE_INIT;

const int MAX_X = 400;
const int MAX_Y = 300;


int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR CmdLine, int ShowCmd) {

	MSG msg;
   WNDCLASSEX wc;
   wc.cbSize = sizeof(WNDCLASSEX);
   wc.style = CS_VREDRAW | CS_HREDRAW;
   wc.lpfnWndProc = WinProc;
   wc.cbClsExtra = 0;
   wc.cbWndExtra = 0;
   wc.hInstance = hInst;
   wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
   wc.hIconSm = wc.hIcon;
   wc.hCursor = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
   wc.lpszMenuName = NULL;
   wc.lpszClassName = "NucWin";
   RegisterClassEx(&wc);

   hWnd = CreateWindowEx(	NULL,
   								"NucWin",
                           "Simple Window ...ayta",
                           NUCGFX_WINDOWSTYLE_FULLSCREEN,
                           (GetSystemMetrics(SM_CXSCREEN)-WINDOWX)/2,
                           (GetSystemMetrics(SM_CYSCREEN)-WINDOWY)/2,
                           WINDOWX, WINDOWY,
                           NULL,
                           NULL,
                           hInst,
                           NULL );
   ShowWindow(hWnd, ShowCmd);
   UpdateWindow(hWnd);

   Init();
   State = STATE_MAIN;

   while(1) {
   	if(PeekMessage(&msg, NULL, 0,0, PM_REMOVE)) {
      	if(msg.message == WM_QUIT) break;
         TranslateMessage(&msg);
         DispatchMessage(&msg);
      } else {
      	// main loop
         if(State == STATE_MAIN) Main();
      }
   }
   return msg.wParam;
}

LRESULT CALLBACK WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {

	switch(msg) {
	case WM_KEYDOWN:
   	if(wParam == VK_ESCAPE) {
      	PostMessage(hWnd, WM_CLOSE, 0, 0);
         return 0;
      }
	break;

	case WM_CLOSE:
   		State = STATE_QUIT;
   		CleanUp();
   		DestroyWindow(hWnd);
		return 0;
	break;

	case WM_DESTROY:
   		PostQuitMessage(0);
	break;

	default: break;
	}

	return DefWindowProc(hWnd, msg, wParam, lParam);
}

NucGFX *gfx;
NucGFXSurface *surf1, *surf2, *surf;

void Init() 
{
	gfx = new NucGFX(SizeX, SizeY, BPP, NUCGFX_FULLSCREEN, 2);
	gfx->CreateSurface(MAX_X, MAX_Y, &surf1);
	gfx->CreateSurface(MAX_X, MAX_Y, &surf2);
	gfx->CreateSurface(MAX_X, MAX_Y, &surf);

	gfx->LoadBMP("cross1.bmp", surf1);
	gfx->LoadBMP("cross2.bmp", surf2);

	ShowCursor(FALSE);

}

void CleanUp() 
{
	gfx->ReleaseSurface(surf1);
	gfx->ReleaseSurface(surf2);
	gfx->ReleaseSurface(surf);
	delete gfx;
}


int count = 0;

void Main() 
{
	
	WORD i, j;
	BYTE r1, r2, g1, g2, b1, b2, r, g, b, *smp1, *smp2;
	DWORD col1, col2;
	long pitch1, pitch2;
	float t;

	gfx->Lock(surf1);
	smp1 = gfx->SurfaceMemory;
	pitch1 = gfx->SurfacePitch;
	
	gfx->Lock(surf2);
	smp2 = gfx->SurfaceMemory;
	pitch2 = gfx->SurfacePitch;

	gfx->Lock(surf);
	//gfx->Lock(gfx->BackBuffer);
	

    // main2

	t = (float)((cos((double)count/4)+1)/2);

	for(j=0; j<MAX_Y; j++)  {
		for(i=0; i<MAX_X; i++)  {
			gfx->GetPixel32(i, j, &col1, smp1, pitch1);

			r1 = (BYTE)((col1 & 0x00ff0000) >> 16);
			g1 = (BYTE)((col1 & 0x0000ff00) >> 8);
			b1 = (BYTE)((col1 & 0x000000ff));

			gfx->GetPixel32(i, j, &col2, smp2, pitch2);

			r2 = (BYTE)((col2 & 0x00ff0000) >> 16);
			g2 = (BYTE)((col2 & 0x0000ff00) >> 8);
			b2 = (BYTE)((col2 & 0x000000ff));

			
			r = (BYTE)(((float)r1 * t) + ((float)r2 * (1.0-t)));
			g = (BYTE)(((float)g1 * t) + ((float)g2 * (1.0-t)));
			b = (BYTE)(((float)b1 * t) + ((float)b2 * (1.0-t)));
			
			gfx->PutPixel32(i, j, r, g, b, gfx->SurfaceMemory, gfx->SurfacePitch);
		}
	}

	gfx->UnLock(surf1);
	gfx->UnLock(surf2);
	
	gfx->UnLock(surf);
	gfx->Blt(gfx->BackBuffer, surf);
	gfx->Flip();

	count++;

}
