#define WIN32_LEAN_AND_MEAN
#define WINDOWX	800
#define WINDOWY	600

#define STATE_INIT	0
#define STATE_MAIN	1
#define STATE_QUIT	2

#define SizeX 800
#define SizeY 600
#define BPP 32

#define BN 10

#include <windows.h>
#include <math.h>
#include "nucgfx3.h"

LRESULT CALLBACK WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

void Init();
void Main();
void CleanUp();

HWND hWnd;
int State = STATE_INIT;

const int MAX_X = 800;
const int MAX_Y = 600;
const int width = 15;


int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR CmdLine, int ShowCmd) {

	MSG msg;
	WNDCLASSEX wc;
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_VREDRAW | CS_HREDRAW;
	wc.lpfnWndProc = WinProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInst;
	wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wc.hIconSm = wc.hIcon;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = "NucWin";
	RegisterClassEx(&wc);

	hWnd = CreateWindowEx(	NULL,
							"NucWin",
							"Nina/ASD",
							NUCGFX_WINDOWSTYLE_WINDOWED,
							(GetSystemMetrics(SM_CXSCREEN)-WINDOWX)/2,
							(GetSystemMetrics(SM_CYSCREEN)-WINDOWY)/2,
							WINDOWX, WINDOWY,
							NULL,
							NULL,
							hInst,
							NULL );
	ShowWindow(hWnd, ShowCmd);
	UpdateWindow(hWnd);

	Init();
	State = STATE_MAIN;

	while(1) {
   		if(PeekMessage(&msg, NULL, 0,0, PM_REMOVE)) {
      		if(msg.message == WM_QUIT) break;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		} 
		else  {
      	// main loop
			if(State == STATE_MAIN) Main();
		}
	}
	return msg.wParam;
}

LRESULT CALLBACK WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {

	switch(msg) {
	case WM_KEYDOWN:
   	if(wParam == VK_ESCAPE) {
      	PostMessage(hWnd, WM_CLOSE, 0, 0);
         return 0;
      }
	break;

	case WM_CLOSE:
   		State = STATE_QUIT;
   		CleanUp();
   		DestroyWindow(hWnd);
		return 0;
	break;

	case WM_DESTROY:
   		PostQuitMessage(0);
	break;

	default: break;
	}

	return DefWindowProc(hWnd, msg, wParam, lParam);
}

NucGFX *gfx;
NucGFXSurface *surf;

void Init() {
	gfx = new NucGFX(SizeX, SizeY, BPP, NUCGFX_WINDOWED, 2);
//	gfx->CreateSurface(MAX_X, MAX_Y, &surf);
	ShowCursor(FALSE);
}

void CleanUp() 
{
//	gfx->ReleaseSurface(surf);
	delete gfx;
	ShowCursor(TRUE);
}

int times = 0;

void Main() 
{
	unsigned int i, dir, pos, poscount, col;	
	
	gfx->ClearSurf(0,0,0,gfx->BackBuffer);
	gfx->Lock(gfx->BackBuffer);

    // main2

	//////////  vertical  //////////

	dir = times/SizeX;				// find x direction & position 
	if(dir%2==0)			
		pos = times%SizeX;
	else 
		pos = SizeX -1 - times%SizeX;

	for(poscount=pos-width; poscount<=pos+width; poscount++)  {
		if(poscount>0 && poscount<SizeX)  {
			col = 255-(((pos-poscount)*(pos-poscount)*255)/(width*width));
			for(i=0; i<SizeY; i++)  {
				gfx->PutPixel(poscount, i, 0, 0, col, gfx->FrameBuffer);
			}
		}
	}

	//////////  across  //////////

	dir = times/SizeY;				// find y direction & position 
	if(dir%2==0)			
		pos = times%SizeY;
	else 
		pos = SizeY -1 - times%SizeY;

	for(poscount=pos-width; poscount<=pos+width; poscount++)  {
		if(poscount>0 && poscount<SizeY)  {
			col = 255-(((pos-poscount)*(pos-poscount)*255)/(width*width));
			for(i=0; i<SizeX; i++)  {
				gfx->PutPixel(i, poscount, 0, 0, col, gfx->FrameBuffer);
			}
		}
	}

	//////////////////////////////

	gfx->UnLock(gfx->BackBuffer);
	gfx->Flip();

	times++;
}
