#define WIN32_LEAN_AND_MEAN
#define WINDOWX	400
#define WINDOWY	300

#define STATE_INIT	0
#define STATE_MAIN	1
#define STATE_QUIT	2

#define SizeX 640
#define SizeY 480
#define BPP 32

#define BN 10

#include <windows.h>
#include <math.h>
#include "nucgfx3.h"

LRESULT CALLBACK WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

void Init();
void Main();
void CleanUp();

HWND hWnd;
int State = STATE_INIT;

const int MAX_X = 320;
const int MAX_Y = 240;


int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR CmdLine, int ShowCmd) {

	MSG msg;
   WNDCLASSEX wc;
   wc.cbSize = sizeof(WNDCLASSEX);
   wc.style = CS_VREDRAW | CS_HREDRAW;
   wc.lpfnWndProc = WinProc;
   wc.cbClsExtra = 0;
   wc.cbWndExtra = 0;
   wc.hInstance = hInst;
   wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
   wc.hIconSm = wc.hIcon;
   wc.hCursor = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
   wc.lpszMenuName = NULL;
   wc.lpszClassName = "NucWin";
   RegisterClassEx(&wc);

   hWnd = CreateWindowEx(	NULL,
   								"NucWin",
                           "Simple Window ...ayta",
                           NUCGFX_WINDOWSTYLE_FULLSCREEN,
                           (GetSystemMetrics(SM_CXSCREEN)-WINDOWX)/2,
                           (GetSystemMetrics(SM_CYSCREEN)-WINDOWY)/2,
                           WINDOWX, WINDOWY,
                           NULL,
                           NULL,
                           hInst,
                           NULL );
   ShowWindow(hWnd, ShowCmd);
   UpdateWindow(hWnd);

   Init();
   State = STATE_MAIN;

   while(1) {
   	if(PeekMessage(&msg, NULL, 0,0, PM_REMOVE)) {
      	if(msg.message == WM_QUIT) break;
         TranslateMessage(&msg);
         DispatchMessage(&msg);
      } else {
      	// main loop
         if(State == STATE_MAIN) Main();
      }
   }
   return msg.wParam;
}

LRESULT CALLBACK WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {

	switch(msg) {
	case WM_KEYDOWN:
   	if(wParam == VK_ESCAPE) {
      	PostMessage(hWnd, WM_CLOSE, 0, 0);
         return 0;
      }
	break;

	case WM_CLOSE:
   		State = STATE_QUIT;
   		CleanUp();
   		DestroyWindow(hWnd);
		return 0;
	break;

	case WM_DESTROY:
   		PostQuitMessage(0);
	break;

	default: break;
	}

	return DefWindowProc(hWnd, msg, wParam, lParam);
}

NucGFX *gfx;
NucGFXSurface *surf;

void Init() {
	gfx = new NucGFX(SizeX, SizeY, BPP, NUCGFX_FULLSCREEN, 2);
	gfx->CreateSurface(MAX_X, MAX_Y, &surf);
}

void CleanUp() 
{
	gfx->ReleaseSurface(surf);
	delete gfx;
}

int count = 0;
float q = 40000;


void Main() 
{

	int i, j , k;
	double vo;
	int x[BN], y[BN], d[BN], v[BN];


	// main 2
	
	gfx->ClearSurf(0,0,0,surf);
	gfx->Lock(surf);

	for(k=0; k<BN; k++)  {
		x[k] = (10 + 32*k);
		y[k] = (int)(120 + 100*sin((double)count*k*0.1));
	}
	

	for(j=0; j<MAX_Y; j++)  {
		for(i=0; i<MAX_X; i++)  {		// screen

				vo = 0;
	
				for(k=0; k<BN; k++)  {
					d[k] = (x[k]-i)*(x[k]-i) + (y[k]-j)*(y[k]-j) +1;
					v[k] = (int)(q / d[k]);
					vo+=v[k];
				}

				if(vo>255) vo = 255;
		
			gfx->PutPixel32((WORD)i, (WORD)j, (DWORD)vo, gfx->SurfaceMemory, gfx->SurfacePitch);
			
		}
	}

	gfx->UnLock(surf);
	gfx->Blt(gfx->BackBuffer, surf);
	gfx->Flip();

	count++;
}
