#include "nucgfx3.h"

#define CLIP_C	0x0000
#define CLIP_N	0x0008
#define CLIP_S	0x0004
#define CLIP_E	0x0002
#define CLIP_W	0x0001

#define CLIP_NE 0x000a
#define CLIP_SE 0x0006
#define CLIP_NW 0x0009
#define CLIP_SW 0x0005

#define RGB32(r,g,b) ((b&255) | ((g&255) << 8) | ((r&255) << 16)) //32bit x888

extern int min_clip_x, min_clip_y, max_clip_x, max_clip_y; // manual clipper coords

int ClipLine(int &x1, int &y1, int &x2, int &y2) 
{

	int xc1 = x1;
	int yc1 = y1;
	int xc2 = x2;
	int yc2 = y2;
	int p1_code = 0;
	int p2_code = 0;

	// determine codes for p1 and p2
	if(y1 < min_clip_y) { p1_code |= CLIP_N;
	} else {
		if(y1 > max_clip_y) { p1_code |= CLIP_S;
		} else {
			if(x1 < min_clip_x) { p1_code |= CLIP_W;
			} else {
				if(x1 > max_clip_x) p1_code |= CLIP_E;
			}
		}
	}

	if(y2 < min_clip_y) { p2_code |= CLIP_N;
	} else {
		if(y2 > max_clip_y) { p2_code |= CLIP_S;
		} else {
			if(x2 < min_clip_x) { p2_code |= CLIP_W;
			} else {
				if(x2 > max_clip_x) p2_code |= CLIP_E;
			}
		}
	}

	if((p1_code & p2_code)) return 0; // trivial rejection
	if(p1_code == 0 && p2_code == 0) return 1; // test if totally visible

	// find clip point for p1
	switch(p1_code) {
		case CLIP_C: break;
		
		case CLIP_N:
			yc1 = min_clip_y;
			xc1 = int(x1 + 0.5+(min_clip_y-y1)*(x2-x1) / (y2-y1));
			break;

		case CLIP_S:
			yc1 = max_clip_y;
			xc1 = int(x1 + 0.5+(max_clip_y-y1)*(x2-x1) / (y2-y1));
			break;

		case CLIP_W:
			xc1 = min_clip_x;
			yc1 = int(y1 + 0.5+(min_clip_x-x1)*(y2-y1) / (x2-x1));
			break;

		case CLIP_E:
			xc1 = max_clip_x;
			yc1 = int(y1 + 0.5+(max_clip_x-x1)*(y2-y1) / (x2-x1));
			break;

		case CLIP_NE:
			yc1 = min_clip_y;
			xc1 = int(x1 + 0.5+(min_clip_y-y1)*(x2-x1) / (y2-y1));
			if(xc1 < min_clip_x || xc1 > max_clip_x) {
				xc1 = max_clip_x;
				yc1 = int(y1 + 0.5+(max_clip_x-x1)*(y2-y1) / (x2-x1));
			}
			break;

		case CLIP_SE:
			yc1 = max_clip_y;
			xc1 = int(x1 + 0.5+(max_clip_y-y1)*(x2-x1) / (y2-y1));
			if(xc1 < min_clip_x || xc1 > max_clip_x) {
				xc1 = max_clip_x;
				yc1 = int(y1 + 0.5+(max_clip_x-x1)*(y2-y1) / (x2-x1));
			}
			break;

		case CLIP_NW:
			yc1 = min_clip_y;
			xc1 = int(x1 + 0.5+(min_clip_y-y1)*(x2-x1) / (y2-y1));
			if(xc1 < min_clip_x || xc1 > max_clip_x) {
				xc1 = min_clip_x;
				yc1 = int(y1 + 0.5+(min_clip_x-x1)*(y2-y1) / (x2-x1));
			}
			break;

		case CLIP_SW:
			yc1 = max_clip_y;
			xc1 = int(x1 + 0.5+(max_clip_y-y1)*(x2-x1) / (y2-y1));
			if(xc1 < min_clip_x || xc1 > max_clip_x) {
				xc1 = min_clip_x;
				yc1 = int(y1 + 0.5+(min_clip_x-x1)*(y2-y1) / (x2-x1));
			}
			break;

		default: break;
	}

	// find clip point for p2
	switch(p2_code) {
		case CLIP_C: break;
		
		case CLIP_N:
			yc2 = min_clip_y;
			xc2 = int(x2 + 0.5+(min_clip_y-y2)*(x1-x2) / (y1-y2));
			break;

		case CLIP_S:
			yc2 = max_clip_y;
			xc2 = int(x2 + 0.5+(max_clip_y-y2)*(x1-x2) / (y1-y2));
			break;

		case CLIP_W:
			xc2 = min_clip_x;
			yc2 = int(y2 + 0.5+(min_clip_x-x2)*(y1-y2) / (x1-x2));
			break;

		case CLIP_E:
			xc2 = max_clip_x;
			yc2 = int(y1 + 0.5+(max_clip_x-x2)*(y1-y2) / (x1-x2));
			break;

		case CLIP_NE:
			yc2 = min_clip_y;
			xc2 = int(x2 + 0.5+(min_clip_y-y2)*(x1-x2) / (y1-y2));
			if(xc2 < min_clip_x || xc2 > max_clip_x) {
				xc2 = max_clip_x;
				yc2 = int(y2 + 0.5+(max_clip_x-x2)*(y1-y2) / (x1-x2));
			}
			break;

		case CLIP_SE:
			yc2 = max_clip_y;
			xc2 = int(x2 + 0.5+(max_clip_y-y2)*(x1-x2) / (y1-y2));
			if(xc2 < min_clip_x || xc2 > max_clip_x) {
				xc2 = max_clip_x;
				yc2 = int(y2 + 0.5+(max_clip_x-x2)*(y1-y2) / (x1-x2));
			}
			break;

		case CLIP_NW:
			yc2 = min_clip_y;
			xc2 = int(x2 + 0.5+(min_clip_y-y2)*(x1-x2) / (y1-y2));
			if(xc2 < min_clip_x || xc2 > max_clip_x) {
				xc2 = min_clip_x;
				yc2 = int(y2 + 0.5+(min_clip_x-x2)*(y1-y2) / (x1-x2));
			}
			break;

		case CLIP_SW:
			yc2 = max_clip_y;
			xc2 = int(x2 + 0.5+(max_clip_y-y2)*(x1-x2) / (y1-y2));
			if(xc2 < min_clip_x || xc2 > max_clip_x) {
				xc2 = min_clip_x;
				yc2 = int(y2 + 0.5+(min_clip_x-x2)*(y1-y2) / (x1-x2));
			}
			break;

		default: break;
	}

	// bounds check
	if(	(xc1 < min_clip_x) || (xc1 > max_clip_x) ||
		(yc1 < min_clip_y) || (yc1 > max_clip_y) ||
		(xc2 < min_clip_x) || (xc2 > max_clip_x) ||
		(yc2 < min_clip_y) || (yc2 > max_clip_x)  ) return 0;

	x1 = xc1;
	y1 = yc1;
	x2 = xc2;
	y2 = yc2;

	return 1;
}


void Line(WORD X1, WORD Y1, WORD X2, WORD Y2, BYTE r, BYTE g, BYTE b, BYTE *buffer, long pitch)
{
	int x1 = (int)X1;
	int x2 = (int)X2;
	int y1 = (int)Y1;
	int y2 = (int)Y2;

	int dx, dy, dx2, dy2;
	int x_inc, y_inc;
	int error;
	int i;

	if(!ClipLine(x1, y1, x2, y2)) 
		return;

	DWORD color = RGB32(r, g, b);

	pitch >>= 2;
	DWORD *ptr = (DWORD*)buffer + y1*pitch+x1;
	dx = (int)x2 - (int)x1;
	dy = (int)y2 - (int)y1;
	// Test direction of line
	if(dx>=0) x_inc = 1; else {x_inc = -1; dx = -dx; }
	if(dy>=0) y_inc = pitch; else {y_inc = -pitch; dy = -dy; }
	dx2 = dx << 1;
	dy2 = dy << 1;

	if(dx > dy) {
		error = dy2-dx;
		for(i=0;i<=dx;i++) {
			if(!(*ptr)) 
				*ptr = color;
			else
				*ptr = 0;
			if(error >=0) {
				error -= dx2;
				ptr += y_inc;
			}
			error += dy2;
			ptr += x_inc;
		}
	} else {
		error = dx2 - dy;
		for(i=0;i<=dy;i++) {
			if(!(*ptr)) 
				*ptr = color;
			else
				*ptr = 0;
			if(error >= 0) {
				error -= dy2;
				ptr += x_inc;
			}
			error += dx2;
			ptr += y_inc;
		}
	}	
	return;
}
