#ifndef _RASTMATH_H_		
#define _RASTMATH_H_

#include <math.h>

class Matrix4x4
{
public:
	float m[4][4];

	Matrix4x4();
	Matrix4x4(Matrix4x4 &mat);

	Matrix4x4 operator +(const Matrix4x4 &mat) const;
	Matrix4x4 operator -(const Matrix4x4 &mat) const;
	Matrix4x4 operator *(const Matrix4x4 &mat) const;

	void operator +=(const Matrix4x4 &mat);
	void operator -=(const Matrix4x4 &mat);
	void operator *=(const Matrix4x4 &mat)
	{  	(*this) = (*this) * mat;  }

	void reset();

	void Translate(float x, float y, float z);
	void Rotate(float x, float y, float z);
	void Scale(float x, float y, float z);
};


class Vector
{
public:
	float x, y, z;

	inline Vector()
	{  x = y = z = 0;  }
	inline Vector(const Vector &vect)
	{  x = vect.x; y = vect.y; z = vect.z;  }
	inline Vector(float X, float Y, float Z)
	{  x = X; y = Y; z = Z;  }

	inline Vector operator +(const Vector &vect) const 
	{  return Vector(x+vect.x, y+vect.y, z+vect.z);  }
	inline Vector operator -(const Vector &vect) const
	{  return Vector(x-vect.x, y-vect.y, z-vect.z);  }

	inline void operator +=(const Vector &vect)
	{  x += vect.x; y += vect.y;  z += vect.z;  }
	inline void operator -=(const Vector &vect)
	{  x -= vect.x; y -= vect.y;  z -= vect.z;  }
	
	inline Vector crossproduct(const Vector &v2)  const // the crossproduct will make you happy
	{  return Vector(y*v2.z-z*v2.y, z*v2.x-x*v2.z, x*v2.y-y*v2.x);  }
	inline float dotproduct(const Vector &v2) const// the dotproduct will make you believe
	{  return x*v2.x + y*v2.y + z*v2.z;  }

	inline float length() const
	{  return float(sqrt(double(x*x+y*y+z*z)));  }

	void transform(const Matrix4x4 &m);

	void Normalize();
	
};


#endif
