#define WIN32_LEAN_AND_MEAN

#define WINDOWX	800
#define WINDOWY	600

#define STATE_INIT	0
#define STATE_MAIN	1
#define STATE_QUIT	2

#define SizeX 800
#define SizeY 600

#define SizeXs 640
#define SizeYs 480

#define ScreenX 400
#define ScreenY 300

#define BPP 32
#define SCR 300

#include <windows.h>
#include <math.h>
#include "nucgfx3.h"
#include "fmod.h"
#include "rastmath.h"
#include "triangle.h"

LRESULT CALLBACK WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

void CalculateParticle();
void initparticle();
void addparticle (int x,int y); 
void clearall();

void Line(WORD X1, WORD Y1, WORD X2, WORD Y2, BYTE r, BYTE g, BYTE b, BYTE *buffer, long pitch);
int ClipLine(int &x1, int &y1, int &x2, int &y2);

void Project(Vertex *vertex); 
void DrawFlat(Triangle *tr, BYTE *buffer, long pitch);

void ScanLine(int y, int x1, int x2);

NucGFX *gfx;

void Init();
void Main();
void CleanUp();

HWND hWnd;
int State = STATE_INIT;

const int particlesize = 40;

const int MAX_X = 800;
const int MAX_Y = 600;
const int XRes = 800;
const int YRes = 600;
const float pi = 3.1415926535f;
const float quarter_pi = pi/4.0f;


int min_clip_x, min_clip_y, max_clip_x, max_clip_y; // manual clipper coords

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR CmdLine, int ShowCmd) {

	MSG msg;
	WNDCLASSEX wc;
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_VREDRAW | CS_HREDRAW;
	wc.lpfnWndProc = WinProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInst;
	wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wc.hIconSm = wc.hIcon;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = "NucWin";
	RegisterClassEx(&wc);

	hWnd = CreateWindowEx(	NULL,
							"NucWin",
							"Nina/ASD",
							NUCGFX_WINDOWSTYLE_FULLSCREEN,
							(GetSystemMetrics(SM_CXSCREEN)-WINDOWX)/2,
							(GetSystemMetrics(SM_CYSCREEN)-WINDOWY)/2,
							WINDOWX, WINDOWY,
							NULL,
							NULL,
							hInst,
							NULL );
	ShowWindow(hWnd, ShowCmd);
	UpdateWindow(hWnd);

	Init();
	State = STATE_MAIN;

	while(1) {
   		if(PeekMessage(&msg, NULL, 0,0, PM_REMOVE)) {
      		if(msg.message == WM_QUIT) break;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		} 
		else  {
      	// main loop
			if(State == STATE_MAIN) Main();
		}
	}
	return msg.wParam;
}

LRESULT CALLBACK WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {

	switch(msg) {
	case WM_KEYDOWN:
   	if(wParam == VK_ESCAPE) {
      	PostMessage(hWnd, WM_CLOSE, 0, 0);
         return 0;
      }
	break;

	case WM_CLOSE:
   		State = STATE_QUIT;
   		CleanUp();
   		DestroyWindow(hWnd);
		return 0;
	break;

	case WM_DESTROY:
   		PostQuitMessage(0);
	break;

	default: break;
	}

	return DefWindowProc(hWnd, msg, wParam, lParam);
}

NucGFXSurface *surf1, *surf2, *surf, *surfc, *surfm, *surf3, *wobsurf; 
NucGFXSurface *initsurf, *fadesurf, *tempsurf, *scrollsurf, *initsurf2, *midsurf;
Vertex vert[8];				// cube vertices
Matrix4x4 wcstm;			// world coordinate system transformation matrix
Matrix4x4 wcstm2;			// world coordinate system transformation matrix
Matrix4x4 wcstm3;			// world coordinate system transformation matrix
Matrix4x4 wcstm4;			// world coordinate system transformation matrix
Matrix4x4 wcstm5;			// world coordinate system transformation matrix
Matrix4x4 wcstm6;			// world coordinate system transformation matrix
Matrix4x4 wcstrans;			// world coordinate system translation matrix
Matrix4x4 wcstrans2;			// world coordinate system translation matrix
Matrix4x4 wcstrans3;			// world coordinate system translation matrix
Matrix4x4 wcstrans4;			// world coordinate system translation matrix
Matrix4x4 wcstrans5;			// world coordinate system translation matrix
Matrix4x4 wcstrans6;			// world coordinate system translation matrix
Vertex pervert[8];				// salooooooooooooooooooooooo
Triangle cube[12];
Vertex pervert2[8];				// salooooooooooooooooooooooo
Triangle cube2[12];
Vertex pervert3[8];				// salooooooooooooooooooooooo
Triangle cube3[12];
Vertex pervert4[8];				// salooooooooooooooooooooooo
Triangle cube4[12];
Vertex pervert5[8];				// salooooooooooooooooooooooo
Triangle cube5[12];
Vertex pervert6[8];				// salooooooooooooooooooooooo
Triangle cube6[12];
int *LeftX, *RightX;
float FOV = quarter_pi;
float dist = (float)WINDOWX/(float)(2*tan(FOV/2));
DWORD StartTime;
Vector Normal(0, 0, 1);
BYTE *nexmem;		// for the woble
long nexpitch;		// for the woble

FMUSIC_MODULE *module;

void Init() 
{
	initparticle();

	RECT topr, bottomr;		// for the scroller

	gfx = new NucGFX(SizeX, SizeY, BPP, NUCGFX_FULLSCREEN, 2);
	gfx->CreateSurface(400, 300, &surf1);	// crossfader
	gfx->CreateSurface(400, 300, &surf2);	// crossfader
	gfx->CreateSurface(400, 300, &surf3);	// crossfader
	gfx->CreateSurface(400, 300, &surfc);	// crossfader
	gfx->CreateSurface(300, 300, &wobsurf); 
	gfx->CreateSurface(400, 600, &surf);	// scroller
	gfx->CreateSurface(400, 1200, &scrollsurf);
	gfx->CreateSurface(SizeXs, SizeYs, &surfm);	// moire
	gfx->CreateSurface(400, 300, &initsurf);	// initial gfx
	gfx->CreateSurface(400, 300, &initsurf2);	// initial gfx sweep
	gfx->CreateSurface(400, 300, &fadesurf);		// fader
	gfx->CreateSurface(400, 300, &tempsurf);	// fader
	gfx->CreateSurface(800, 600, &midsurf);
	gfx->LoadJPEG("scroller.jpg", surf);	// scroller
	gfx->LoadJPEG("react2.jpg", surf1);		// crossfader
	gfx->LoadJPEG("react3.jpg", surf2);		// crossfader
	gfx->LoadJPEG("react1.jpg", surf3);		// crossfader
	gfx->LoadJPEG("invites.jpg", initsurf);

	ShowCursor(FALSE);

	// line clipping for the moire pattern
	min_clip_x = 0;		min_clip_y = 0;
	max_clip_x = XRes;	max_clip_y = YRes;
	
	///////////////////////////////////////

	///////////   raster   ///////////////
	LeftX = new int[WINDOWY];		// edge arrays
	RightX = new int[WINDOWY];

	vert[0] = Vertex(1,1,-1);			// local coordinates
	vert[1] = Vertex(1,-1,-1);
	vert[2] = Vertex(-1,-1,-1);
	vert[3] = Vertex(-1,1,-1);
	vert[4] = Vertex(1,1,1);
	vert[5] = Vertex(1,-1,1);
	vert[6] = Vertex(-1,-1,1);
	vert[7] = Vertex(-1,1,1);

	for(int i=0; i<8; i++)
		vert[i].SetColor(0, 0, 255);

	cube[0] = Triangle(&pervert[0], &pervert[1], &pervert[2]);
	cube[1] = Triangle(&pervert[0], &pervert[2], &pervert[3]);
	cube[2] = Triangle(&pervert[3], &pervert[2], &pervert[6]);
	cube[3] = Triangle(&pervert[3], &pervert[6], &pervert[7]);
	cube[4] = Triangle(&pervert[4], &pervert[7], &pervert[6]);
	cube[5] = Triangle(&pervert[4], &pervert[6], &pervert[5]);
	cube[6] = Triangle(&pervert[0], &pervert[4], &pervert[5]);
	cube[7] = Triangle(&pervert[0], &pervert[5], &pervert[1]);
	cube[8] = Triangle(&pervert[7], &pervert[4], &pervert[0]);
	cube[9] = Triangle(&pervert[7], &pervert[0], &pervert[3]);
	cube[10] = Triangle(&pervert[5], &pervert[6], &pervert[2]);
	cube[11] = Triangle(&pervert[5], &pervert[2], &pervert[1]);

	cube2[0] = Triangle(&pervert2[0], &pervert2[1], &pervert2[2]);
	cube2[1] = Triangle(&pervert2[0], &pervert2[2], &pervert2[3]);
	cube2[2] = Triangle(&pervert2[3], &pervert2[2], &pervert2[6]);
	cube2[3] = Triangle(&pervert2[3], &pervert2[6], &pervert2[7]);
	cube2[4] = Triangle(&pervert2[4], &pervert2[7], &pervert2[6]);
	cube2[5] = Triangle(&pervert2[4], &pervert2[6], &pervert2[5]);
	cube2[6] = Triangle(&pervert2[0], &pervert2[4], &pervert2[5]);
	cube2[7] = Triangle(&pervert2[0], &pervert2[5], &pervert2[1]);
	cube2[8] = Triangle(&pervert2[7], &pervert2[4], &pervert2[0]);
	cube2[9] = Triangle(&pervert2[7], &pervert2[0], &pervert2[3]);
	cube2[10] = Triangle(&pervert2[5], &pervert2[6], &pervert2[2]);
	cube2[11] = Triangle(&pervert2[5], &pervert2[2], &pervert2[1]);

	cube3[0] = Triangle(&pervert3[0], &pervert3[1], &pervert3[2]);
	cube3[1] = Triangle(&pervert3[0], &pervert3[2], &pervert3[3]);
	cube3[2] = Triangle(&pervert3[3], &pervert3[2], &pervert3[6]);
	cube3[3] = Triangle(&pervert3[3], &pervert3[6], &pervert3[7]);
	cube3[4] = Triangle(&pervert3[4], &pervert3[7], &pervert3[6]);
	cube3[5] = Triangle(&pervert3[4], &pervert3[6], &pervert3[5]);
	cube3[6] = Triangle(&pervert3[0], &pervert3[4], &pervert3[5]);
	cube3[7] = Triangle(&pervert3[0], &pervert3[5], &pervert3[1]);
	cube3[8] = Triangle(&pervert3[7], &pervert3[4], &pervert3[0]);
	cube3[9] = Triangle(&pervert3[7], &pervert3[0], &pervert3[3]);
	cube3[10] = Triangle(&pervert3[5], &pervert3[6], &pervert3[2]);
	cube3[11] = Triangle(&pervert3[5], &pervert3[2], &pervert3[1]);

	cube4[0] = Triangle(&pervert4[0], &pervert4[1], &pervert4[2]);
	cube4[1] = Triangle(&pervert4[0], &pervert4[2], &pervert4[3]);
	cube4[2] = Triangle(&pervert4[3], &pervert4[2], &pervert4[6]);
	cube4[3] = Triangle(&pervert4[3], &pervert4[6], &pervert4[7]);
	cube4[4] = Triangle(&pervert4[4], &pervert4[7], &pervert4[6]);
	cube4[5] = Triangle(&pervert4[4], &pervert4[6], &pervert4[5]);
	cube4[6] = Triangle(&pervert4[0], &pervert4[4], &pervert4[5]);
	cube4[7] = Triangle(&pervert4[0], &pervert4[5], &pervert4[1]);
	cube4[8] = Triangle(&pervert4[7], &pervert4[4], &pervert4[0]);
	cube4[9] = Triangle(&pervert4[7], &pervert4[0], &pervert4[3]);
	cube4[10] = Triangle(&pervert4[5], &pervert4[6], &pervert4[2]);
	cube4[11] = Triangle(&pervert4[5], &pervert4[2], &pervert4[1]);

	cube5[0] = Triangle(&pervert5[0], &pervert5[1], &pervert5[2]);
	cube5[1] = Triangle(&pervert5[0], &pervert5[2], &pervert5[3]);
	cube5[2] = Triangle(&pervert5[3], &pervert5[2], &pervert5[6]);
	cube5[3] = Triangle(&pervert5[3], &pervert5[6], &pervert5[7]);
	cube5[4] = Triangle(&pervert5[4], &pervert5[7], &pervert5[6]);
	cube5[5] = Triangle(&pervert5[4], &pervert5[6], &pervert5[5]);
	cube5[6] = Triangle(&pervert5[0], &pervert5[4], &pervert5[5]);
	cube5[7] = Triangle(&pervert5[0], &pervert5[5], &pervert5[1]);
	cube5[8] = Triangle(&pervert5[7], &pervert5[4], &pervert5[0]);
	cube5[9] = Triangle(&pervert5[7], &pervert5[0], &pervert5[3]);
	cube5[10] = Triangle(&pervert5[5], &pervert5[6], &pervert5[2]);
	cube5[11] = Triangle(&pervert5[5], &pervert5[2], &pervert5[1]);

	cube6[0] = Triangle(&pervert6[0], &pervert6[1], &pervert6[2]);
	cube6[1] = Triangle(&pervert6[0], &pervert6[2], &pervert6[3]);
	cube6[2] = Triangle(&pervert6[3], &pervert6[2], &pervert6[6]);
	cube6[3] = Triangle(&pervert6[3], &pervert6[6], &pervert6[7]);
	cube6[4] = Triangle(&pervert6[4], &pervert6[7], &pervert6[6]);
	cube6[5] = Triangle(&pervert6[4], &pervert6[6], &pervert6[5]);
	cube6[6] = Triangle(&pervert6[0], &pervert6[4], &pervert6[5]);
	cube6[7] = Triangle(&pervert6[0], &pervert6[5], &pervert6[1]);
	cube6[8] = Triangle(&pervert6[7], &pervert6[4], &pervert6[0]);
	cube6[9] = Triangle(&pervert6[7], &pervert6[0], &pervert6[3]);
	cube6[10] = Triangle(&pervert6[5], &pervert6[6], &pervert6[2]);
	cube6[11] = Triangle(&pervert6[5], &pervert6[2], &pervert6[1]);

//	for(int g=0; g<12; g++)
//		cube[g].SetColor(0,150,150);

	cube[0].SetColor(150,0,250);
	cube[1].SetColor(150,0,250);
	cube[2].SetColor(0,50,250);
	cube[3].SetColor(0,50,250);
	cube[4].SetColor(0,100,250);
	cube[5].SetColor(0,100,250);
	cube[6].SetColor(0,150,250);
	cube[7].SetColor(0,150,250);
	cube[8].SetColor(50,0,250);
	cube[9].SetColor(50,0,250);
	cube[10].SetColor(100,0,250);
	cube[11].SetColor(100,0,250);

	cube2[0].SetColor(150,0,250);
	cube2[1].SetColor(150,0,250);
	cube2[2].SetColor(0,50,250);
	cube2[3].SetColor(0,50,250);
	cube2[4].SetColor(0,100,250);
	cube2[5].SetColor(0,100,250);
	cube2[6].SetColor(0,150,250);
	cube2[7].SetColor(0,150,250);
	cube2[8].SetColor(50,0,250);
	cube2[9].SetColor(50,0,250);
	cube2[10].SetColor(100,0,250);
	cube2[11].SetColor(100,0,250);

	cube3[0].SetColor(150,0,250);
	cube3[1].SetColor(150,0,250);
	cube3[2].SetColor(0,50,250);
	cube3[3].SetColor(0,50,250);
	cube3[4].SetColor(0,100,250);
	cube3[5].SetColor(0,100,250);
	cube3[6].SetColor(0,150,250);
	cube3[7].SetColor(0,150,250);
	cube3[8].SetColor(50,0,250);
	cube3[9].SetColor(50,0,250);
	cube3[10].SetColor(100,0,250);
	cube3[11].SetColor(100,0,250);

	cube4[0].SetColor(150,0,250);
	cube4[1].SetColor(150,0,250);
	cube4[2].SetColor(0,50,250);
	cube4[3].SetColor(0,50,250);
	cube4[4].SetColor(0,100,250);
	cube4[5].SetColor(0,100,250);
	cube4[6].SetColor(0,150,250);
	cube4[7].SetColor(0,150,250);
	cube4[8].SetColor(50,0,250);
	cube4[9].SetColor(50,0,250);
	cube4[10].SetColor(100,0,250);
	cube4[11].SetColor(100,0,250);
	
	cube5[0].SetColor(150,0,250);
	cube5[1].SetColor(150,0,250);
	cube5[2].SetColor(0,50,250);
	cube5[3].SetColor(0,50,250);
	cube5[4].SetColor(0,100,250);
	cube5[5].SetColor(0,100,250);
	cube5[6].SetColor(0,150,250);
	cube5[7].SetColor(0,150,250);
	cube5[8].SetColor(50,0,250);
	cube5[9].SetColor(50,0,250);
	cube5[10].SetColor(100,0,250);
	cube5[11].SetColor(100,0,250);
	
	cube6[0].SetColor(150,0,250);
	cube6[1].SetColor(150,0,250);
	cube6[2].SetColor(0,50,250);
	cube6[3].SetColor(0,50,250);
	cube6[4].SetColor(0,100,250);
	cube6[5].SetColor(0,100,250);
	cube6[6].SetColor(0,150,250);
	cube6[7].SetColor(0,150,250);
	cube6[8].SetColor(50,0,250);
	cube6[9].SetColor(50,0,250);
	cube6[10].SetColor(100,0,250);
	cube6[11].SetColor(100,0,250);

	wcstrans.Translate(-2, 4, 20);
	wcstrans2.Translate(-4, 0, 20);
	wcstrans3.Translate(2, 0, 20);
	wcstrans4.Translate(-6, -4, 20);
	wcstrans5.Translate(0, -4, 20);
	wcstrans6.Translate(6, -4, 20);

	//////////////  scroller trick  ///////////////////
	
	gfx->ClearSurf(0, 0, 0, fadesurf);

	topr.top = 0;
	topr.left = 0;
	topr.right = 400;
	topr.bottom = 300;
	
	bottomr.top = 900;
	bottomr.left = 0;
	bottomr.right = 400;
	bottomr.bottom = 1200;

	gfx->Blt(topr, scrollsurf, topr, fadesurf);
	gfx->Blt(bottomr, scrollsurf, topr, fadesurf);

	topr.bottom = 600;

	bottomr.top = 300;
	bottomr.bottom = 900;
	
	gfx->Blt(bottomr, scrollsurf, topr, surf);

	///////////////////////////////////////////////////

	// initialize FMOD
	FSOUND_SetHWND(hWnd);
	FSOUND_Init(44100, 32, FSOUND_INIT_GLOBALFOCUS);
	module = FMUSIC_LoadSong("<filename here>");
	FMUSIC_SetMasterVolume(module, 100);

	FMUSIC_PlaySong(module);
	
	StartTime = GetTickCount();
}

void CleanUp() 
{
	FMUSIC_StopSong(module);
	FMUSIC_FreeSong(module);
	FSOUND_Close();

	gfx->ReleaseSurface(surf1);
	gfx->ReleaseSurface(surf2);
	gfx->ReleaseSurface(surfc);
	gfx->ReleaseSurface(surf3);
	gfx->ReleaseSurface(surf);
	gfx->ReleaseSurface(surfm);
	gfx->ReleaseSurface(wobsurf);
	gfx->ReleaseSurface(initsurf);
	gfx->ReleaseSurface(initsurf2);
	gfx->ReleaseSurface(fadesurf);
	gfx->ReleaseSurface(tempsurf);
	gfx->ReleaseSurface(scrollsurf);
	gfx->ReleaseSurface(midsurf);
	delete gfx;
	delete [] LeftX;
	delete [] RightX;

	ShowCursor(TRUE);
}

// time

const unsigned int InviteMoire = 250;
const unsigned int MoirePart = 351; 
const unsigned int ParticleEnd = 455;
const unsigned int CrossStart = ParticleEnd+1;
const unsigned int CrossNexus = 572;
const unsigned int NexusMid = 750;
const unsigned int MidRast = 850;
const unsigned int RastScroll = 1200;


//////////////////////////////////////////////////////////////////////
/////////////////////   m a i n   ////////////////////////////////////
//////////////////////////////////////////////////////////////////////

// unsigned int counter = 0;
float angle = 0;

void Main() 
{
	unsigned int counter = (int)(0.02*(GetTickCount() - StartTime));

	gfx->ClearSurf(0,0,0,gfx->BackBuffer);
	
	////////////////////  asd invites  //////////////////

	if(counter<InviteMoire)  {
	
		if (counter<=200)  {
			
			WORD swx, swy, swi;
			BYTE *swmem;
			DWORD swcol;
			long swpitch;

			gfx->Lock(initsurf);
			swmem = gfx->SurfaceMemory;
			swpitch = gfx->SurfacePitch;
			
			gfx->Lock(initsurf2);

			for(swx=0; swx < counter*2; swx++)  {
				for(swy=0; swy<300; swy++)  {
					gfx->GetPixel32(swx, swy, &swcol, swmem, swpitch);
					gfx->PutPixel32(swx, swy, swcol, gfx->SurfaceMemory, gfx->SurfacePitch);
				}
			}
			
			for(swy=0; swy<300; swy++)  {
				gfx->GetPixel32(swx, swy, &swcol, swmem, swpitch);
				for(swi=swx; swi<400; swi++)  {
					gfx->PutPixel32(swi, swy, swcol, gfx->SurfaceMemory, gfx->SurfacePitch);
				}
			}

			gfx->UnLock(initsurf);
			gfx->UnLock(initsurf2);

			gfx->Blt(gfx->BackBuffer, initsurf2);
		}

		if(counter>200)  {

			WORD k, l;
			BYTE ra, rb, ga, gb, ba, bb, r, g, b, *mem1, *mem2;
			DWORD color1, color2;
			long pit1, pit2;
			float tt;
			
			tt = 1.0f - ((float)counter-200.0f)/50.0f;

			gfx->ClearSurf(0, 0, 0, fadesurf);
//			gfx->ClearSurf(0, tempsurf);

			gfx->Lock(initsurf);
			mem1 = gfx->SurfaceMemory;
			pit1 = gfx->SurfacePitch;
		
			gfx->Lock(fadesurf);
			mem2 = gfx->SurfaceMemory;
			pit2 = gfx->SurfacePitch;

			gfx->Lock(tempsurf);

			for(l=0; l<300; l++)  {
				for(k=0; k<400; k++)  {
					gfx->GetPixel32(k, l, &color1, mem1, pit1);
	
					ra = (BYTE)((color1 & 0x00ff0000) >> 16);
					ga = (BYTE)((color1 & 0x0000ff00) >> 8);
					ba = (BYTE)((color1 & 0x000000ff));

					gfx->GetPixel32(k, l, &color2, mem2, pit2);

					rb = (BYTE)((color2 & 0x00ff0000) >> 16);
					gb = (BYTE)((color2 & 0x0000ff00) >> 8);
					bb = (BYTE)((color2 & 0x000000ff));

			
					r = (BYTE)(((float)ra * tt) + ((float)rb * (1.0-tt)));
					g = (BYTE)(((float)ga * tt) + ((float)gb * (1.0-tt)));
					b = (BYTE)(((float)ba * tt) + ((float)bb * (1.0-tt)));
				
					gfx->PutPixel32(k, l, r, g, b, gfx->SurfaceMemory, gfx->SurfacePitch);
				}
			}

			gfx->UnLock(initsurf);
			gfx->UnLock(fadesurf);
			gfx->UnLock(tempsurf);
	
			gfx->Blt(gfx->BackBuffer, tempsurf);
		}
	}

	//////////////////// moire pattern  ////////////////

	if((counter>=InviteMoire)&&(counter<MoirePart))  {

		BYTE r=0, g=0, b=0;

		if((counter-InviteMoire)<=50)  {
			r = (counter-InviteMoire)*3;
			g = 200 - (counter-InviteMoire)*4;
			b = 250;
		}
		else  {
			r = 150 - (counter-InviteMoire-50)*3;
			g = 0;
			b = 250 - (counter-InviteMoire-50) * 5;
		}

		gfx->ClearSurf(0,0,0,surfm);

		gfx->Lock(surfm);

		for(int x=0; x<SizeXs; x++)  {
			Line(x, 0, SizeXs-x-1, SizeYs-1, r, g, b, gfx->SurfaceMemory, gfx->SurfacePitch);
		}

		for(int y=0; y<SizeYs; y++)  {
			Line(0, y, SizeXs-1, SizeYs-y-1, r, g, b, gfx->SurfaceMemory, gfx->SurfacePitch);
		}

		gfx->UnLock(surfm);
		gfx->Blt(gfx->BackBuffer, surfm);

	}	//endif moire


	///////////////////  particles  /////////////////////
	
	if((counter>=MoirePart)&&(counter<ParticleEnd))  {
	
		clearall();

		for (int k=0;k<80;k++)  { 
			double xx=300+280*sin(double(k+counter%800)/11);
			double yy=300+180*sin(double(k+counter%800)/14);
			addparticle((int)xx,(int)yy);
		}
		
		CalculateParticle();	
	}	// endif particles


	if(counter==CrossStart)
		gfx->ClearSurf(0,0,0,gfx->BackBuffer);


	/////////////////////  crossfader  //////////////////

	WORD i, j;
	BYTE r1, r2, g1, g2, b1, b2, r, g, b, *smp1, *smp2;
	DWORD col1, col2;
	long pitch1, pitch2;
	float t;
	
	if((counter>CrossStart)&&(counter<CrossNexus))  {

		t = (float)((cos((double)(counter-CrossStart-1)*0.25)+1)/2);
			
		gfx->Lock(surf1);
		smp1 = gfx->SurfaceMemory;
		pitch1 = gfx->SurfacePitch;
	
		if (counter<508)  {
			gfx->Lock(surf2);
			smp2 = gfx->SurfaceMemory;
			pitch2 = gfx->SurfacePitch;
		}
		if(counter>=508)  {
			gfx->Lock(surf3);
			smp2 = gfx->SurfaceMemory;
			pitch2 = gfx->SurfacePitch;
		}
		
		gfx->Lock(surfc);

		for(j=0; j<300; j++)  {
			for(i=0; i<400; i++)  {
				gfx->GetPixel32(i, j, &col1, smp1, pitch1);
	
				r1 = (BYTE)((col1 & 0x00ff0000) >> 16);
				g1 = (BYTE)((col1 & 0x0000ff00) >> 8);
				b1 = (BYTE)((col1 & 0x000000ff));

				gfx->GetPixel32(i, j, &col2, smp2, pitch2);

				r2 = (BYTE)((col2 & 0x00ff0000) >> 16);
				g2 = (BYTE)((col2 & 0x0000ff00) >> 8);
				b2 = (BYTE)((col2 & 0x000000ff));

			
				r = (BYTE)(r2 + (BYTE)((float)(r1 - r2) * t));
				g = (BYTE)(g2 + (BYTE)((float)(g1 - g2) * t));
				b = (BYTE)(b2 + (BYTE)((float)(b1 - b2) * t));
				
				gfx->PutPixel32(i, j, r, g, b, gfx->SurfaceMemory, gfx->SurfacePitch);
			}
		}

		gfx->UnLock(surf1);
		gfx->UnLock(surf2);
		gfx->UnLock(surf3);
		gfx->UnLock(surfc);
		gfx->Blt(gfx->BackBuffer, surfc);

	}		// endif crossfader


	/////////////  nexus - dont-know-how-its-called effect  //////

	if((counter>=CrossNexus)&&(counter<MidRast))  {
		int x1, x2, y;
		double xx, yy;

		gfx->Lock(surf3);
		nexmem = gfx->SurfaceMemory;
		nexpitch = gfx->SurfacePitch;

		gfx->Lock(wobsurf);

		for(y=0; y<300; y++)  {
			xx=0.03+0.026*sin(double(counter-CrossNexus)*0.04);
			yy=0.08+0.079*sin(double(counter-CrossNexus)*0.0545);
			x1 = (int)(30+(sin(double(y)*xx*0.3+double(counter-CrossNexus)* 0.1)) * 30);
			x2 = (int)(300-30+(sin(double(y)*yy*0.3+double(counter-CrossNexus)* 0.19)) * 30);
			
			ScanLine(y, x1, x2);
		}
	
		gfx->UnLock(surf3);
		gfx->UnLock(wobsurf);
	
		if(counter<NexusMid)
			gfx->Blt(gfx->BackBuffer, wobsurf);

	}	// endif nexus


	//////////////////////  raster 3d   //////////////////////////

	if((counter>=NexusMid)&&(counter<RastScroll))  {

		unsigned int i;
		Matrix4x4 rot;
		float fact; 

		rot.Rotate(angle, angle, 0);
		wcstm.reset();
		wcstm2.reset();
		wcstm3.reset();
		wcstm4.reset();
		wcstm5.reset();
		wcstm6.reset();

		wcstm *= rot;	
		wcstm2 *= rot;	
		wcstm3 *= rot;	
		wcstm4 *= rot;	
		wcstm5 *= rot;	
		wcstm6 *= rot;	

		wcstm *= wcstrans;	
		wcstm2 *= wcstrans2;	
		wcstm3 *= wcstrans3;	
		wcstm4 *= wcstrans4;	
		wcstm5 *= wcstrans5;	
		wcstm6 *= wcstrans6;	

		memcpy(pervert, vert, 8*sizeof(Vertex));
		memcpy(pervert2, vert, 8*sizeof(Vertex));
		memcpy(pervert3, vert, 8*sizeof(Vertex));
		memcpy(pervert4, vert, 8*sizeof(Vertex));
		memcpy(pervert5, vert, 8*sizeof(Vertex));
		memcpy(pervert6, vert, 8*sizeof(Vertex));

		for(i=0; i<8; i++)  {
			pervert[i].pos.transform(wcstm);
			Project(&pervert[i]);
		}	
		
		for(i=0; i<8; i++)  {
			pervert2[i].pos.transform(wcstm2);
			Project(&pervert2[i]);
		}

		for(i=0; i<8; i++)  {
			pervert3[i].pos.transform(wcstm3);
			Project(&pervert3[i]);
		}

		for(i=0; i<8; i++)  {
			pervert4[i].pos.transform(wcstm4);
			Project(&pervert4[i]);
		}	

		for(i=0; i<8; i++)  {
			pervert5[i].pos.transform(wcstm5);
			Project(&pervert5[i]);
		}	

		for(i=0; i<8; i++)  {
			pervert6[i].pos.transform(wcstm6);
			Project(&pervert6[i]);
		}	

		gfx->ClearSurf(0, gfx->BackBuffer);
		gfx->Lock(gfx->BackBuffer);

		for(i=0; i<12; i++)
			cube[i].CalculateNormal();

		for(i=0; i<12; i++)
			cube2[i].CalculateNormal();

		for(i=0; i<12; i++)
			cube3[i].CalculateNormal();

		for(i=0; i<12; i++)
			cube4[i].CalculateNormal();

  		for(i=0; i<12; i++)
			cube5[i].CalculateNormal();

		for(i=0; i<12; i++)
			cube6[i].CalculateNormal();

		for(i=0; i<12; i++)  {
			fact = cube[i].norm.dotproduct(Normal);
			if(fact>0)
				DrawFlat(&cube[i], gfx->FrameBuffer, gfx->Pitch);
		}

		for(i=0; i<12; i++)  {
			fact = cube2[i].norm.dotproduct(Normal);
			if(fact>0) 
				DrawFlat(&cube2[i], gfx->FrameBuffer, gfx->Pitch);
		}

		for(i=0; i<12; i++)  {
			fact = cube3[i].norm.dotproduct(Normal);
			if(fact>0) 
				DrawFlat(&cube3[i], gfx->FrameBuffer, gfx->Pitch);
		}

		for(i=0; i<12; i++)  {
			fact = cube4[i].norm.dotproduct(Normal);
			if(fact>0)
				DrawFlat(&cube4[i], gfx->FrameBuffer, gfx->Pitch);
		}

		for(i=0; i<12; i++)  {
			fact = cube5[i].norm.dotproduct(Normal);
			if(fact>0)
				DrawFlat(&cube5[i], gfx->FrameBuffer, gfx->Pitch);
		}

		for(i=0; i<12; i++)  {
			fact = cube6[i].norm.dotproduct(Normal);
			if(fact>0)
				DrawFlat(&cube6[i], gfx->FrameBuffer, gfx->Pitch);
		}

		gfx->UnLock(gfx->BackBuffer);

		angle = (float)(GetTickCount() - StartTime) / 1000.0f;
	
	}

	//////////////////////   middle   ////////////////////////////

	if((counter>=NexusMid)&&(counter<MidRast))  {
		
		int midx, midy, rad;
		float distsq;
		DWORD midcol;
		
		rad = (counter-NexusMid)*5;

		gfx->Blt(midsurf, wobsurf);
		gfx->Lock(midsurf);
		gfx->Lock(gfx->BackBuffer);

		for(midy=0; midy<600; midy++)  {
			for(midx=0; midx<800; midx++)  {
				distsq = (float)((midx-400)*(midx-400)+(midy-300)*(midy-300));
				if(distsq>rad*rad)  {
					gfx->GetPixel32(midx, midy, &midcol, gfx->SurfaceMemory, gfx->SurfacePitch);
					gfx->PutPixel32(midx, midy, midcol, gfx->FrameBuffer, gfx->Pitch);
				}
			}
		}

		gfx->UnLock(midsurf);
		gfx->UnLock(gfx->BackBuffer);
	}

	///////////////////////  scroller  //////////////////////

	if(counter>=RastScroll)  {
		
		RECT desr, bbr;

		desr.left = 0;
		desr.right = 400;
		desr.top = (counter-RastScroll) % (1200 - SCR);
		desr.bottom = desr.top + SCR;
	
		bbr.left = 0;
		bbr.right = SizeX;
		bbr.top = 0;
		bbr.bottom = SizeY;

		gfx->Blt(bbr, gfx->BackBuffer, desr, scrollsurf);
	}		// endif scroller


	//////////////////////////////////////////////////////////

	gfx->Flip();

//	counter++;
}


/////////////////////////////////////////////////////////////////
/////////////////////   drawflat   //////////////////////////////
/////////////////////////////////////////////////////////////////


void DrawFlat(Triangle *tr, BYTE *buffer, long pitch)
{
	int scanlines, i, j, starty, startx;
	double d01, d02, d12;
	float dx, dy, slope;
	Vertex *vert1, *vert2;

	d01 = (tr->vert[0]->pos.y-tr->vert[1]->pos.y > 0) ? (tr->vert[0]->pos.y-tr->vert[1]->pos.y) : (tr->vert[1]->pos.y-tr->vert[0]->pos.y);
	d02 = (tr->vert[0]->pos.y-tr->vert[2]->pos.y > 0) ? (tr->vert[0]->pos.y-tr->vert[2]->pos.y) : (tr->vert[2]->pos.y-tr->vert[0]->pos.y);
	d12 = (tr->vert[1]->pos.y-tr->vert[2]->pos.y > 0) ? (tr->vert[1]->pos.y-tr->vert[2]->pos.y) : (tr->vert[2]->pos.y-tr->vert[1]->pos.y);
	
	scanlines = (d01>d02) ? (int)d01 : (int)d02;			// scanlines - greater of d01, d02
	scanlines = (scanlines>d12) ? scanlines : (int)d12;

	_asm {
		mov edi, LeftX
		mov ecx, WINDOWY
		mov eax, WINDOWX-1
		rep stosd

		mov edi, RightX
		mov ecx, WINDOWY
		xor eax, eax
		rep stosd
	}	

	for(j=0; j<3; j++)  {
		vert1 = tr->vert[j];
		vert2 = tr->vert[(j+1)%3];
		startx = (int)vert1->pos.x;
		int starty;
		int beginx;

		if(vert1->pos.y>vert2->pos.y)  {
			dx = vert1->pos.x - vert2->pos.x;
			dy = vert1->pos.y - vert2->pos.y;
			starty = (int)vert2->pos.y;
			beginx = (int)vert2->pos.x;
		}
		else  {
			dx = vert2->pos.x - vert1->pos.x;
			dy = vert2->pos.y - vert1->pos.y;
			starty = (int)vert1->pos.y;
			beginx = (int)vert1->pos.x;
		}

		slope = dx/dy;

		int ylines = (int)dy;
		for(i=0; i<=ylines; i++)  {
			int calcx = (int)(slope*(float)i) + beginx;
			if(calcx < 0) calcx = 0;
			if(calcx > WINDOWX) calcx = WINDOWX;
			int y = i+starty;
			if(y >= WINDOWY) break;				
			if(y >= 0) {
				if(calcx < LeftX[y])					
					LeftX[y] = calcx;
				if(calcx > RightX[y])					
					RightX[y] = calcx;
			}
		}
	}

	starty = (tr->vert[0]->pos.y<tr->vert[1]->pos.y) ? (int)tr->vert[0]->pos.y : (int)tr->vert[1]->pos.y;
	starty = (starty<tr->vert[2]->pos.y) ? starty : (int)tr->vert[2]->pos.y;

	int endy = starty + scanlines;
	
	for(j=starty; j<=endy; j++)  {
		if(j >= WINDOWY) break;				
		if(j >= 0) {
			for(i=LeftX[j]; i<RightX[j]; i++) {
				gfx->PutPixel((WORD)i, (WORD)j, tr->r, tr->g, tr->b, buffer, pitch);
			}
		}
	}
}

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

void Project(Vertex *vertex) 
{
    vertex->pos.x = (dist*vertex->pos.x / vertex->pos.z) + (WINDOWX >> 1);	
    vertex->pos.y = (WINDOWY >> 1) - (dist*vertex->pos.y / vertex->pos.z);		
}

//////////////////////////////////////////////////////////////
////////////////  scanline for the nexus  ////////////////////

void ScanLine(int y, int x1, int x2)  
{
	double x0 = 0;
	double xstep = 400 / double(x2-x1);
	DWORD col;

	for(int x=x1; x<x2; x++)  {
		gfx->GetPixel32((WORD)x0, y, &col, nexmem, nexpitch);
		gfx->PutPixel32((WORD)x, y, col, gfx->SurfaceMemory, gfx->SurfacePitch);
		x0 +=xstep;

	}
}

////////////////////////////////////////////////////////////////
