#define WIN32_LEAN_AND_MEAN
#define WINDOWX	640
#define WINDOWY	480

#define STATE_INIT	0
#define STATE_MAIN	1
#define STATE_QUIT	2

#define SizeX 640
#define SizeY 480
#define BPP 32

#define BN 10

#include <vector>
#include <windows.h>
#include <math.h>
#include "nucgfx3.h"
#include "rastmath.h"
#include "triangle.h"

void Project(Vertex *vertex); 
void DrawFlat(Triangle *tr, BYTE *buffer, long pitch);
void ClearZBuffer();

NucGFX *gfx;

LRESULT CALLBACK WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

void Init();
void Main();
void CleanUp();

HWND hWnd;
int State = STATE_INIT;

const int MAX_X = 640;
const int MAX_Y = 480;
const float pi = 3.1415926535f;
const float quarter_pi = pi/4.0f;
float xangle, yangle;
const float rotrads = 0.1f;


int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR CmdLine, int ShowCmd) {

	MSG msg;
	WNDCLASSEX wc;
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_VREDRAW | CS_HREDRAW;
	wc.lpfnWndProc = WinProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInst;
	wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wc.hIconSm = wc.hIcon;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = "NucWin";
	RegisterClassEx(&wc);

	hWnd = CreateWindowEx(	NULL,
							"NucWin",
							"Nina/ASD",
							NUCGFX_WINDOWSTYLE_WINDOWED,
							(GetSystemMetrics(SM_CXSCREEN)-WINDOWX)/2,
							(GetSystemMetrics(SM_CYSCREEN)-WINDOWY)/2,
							WINDOWX, WINDOWY,
							NULL,
							NULL,
							hInst,
							NULL );
	ShowWindow(hWnd, ShowCmd);
	UpdateWindow(hWnd);

	Init();
	State = STATE_MAIN;

	while(1) {
   		if(PeekMessage(&msg, NULL, 0,0, PM_REMOVE)) {
      		if(msg.message == WM_QUIT) break;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		} 
		else  {
      	// main loop
			if(State == STATE_MAIN) Main();
		}
	}
	return msg.wParam;
}

//** debug stuff
int MouseX, MouseY;
//** eod


LRESULT CALLBACK WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {

	switch(msg) {
	case WM_KEYDOWN:
   	if(wParam == VK_ESCAPE) {
      	PostMessage(hWnd, WM_CLOSE, 0, 0);
         return 0;
    }
	if(wParam == VK_LEFT) {
		yangle -= rotrads;
		return 0;
	}
	if(wParam == VK_RIGHT) {
		yangle += rotrads;
		return 0;
	}
	if(wParam == VK_UP) {
		xangle -= rotrads;
		return 0;
	}
	if(wParam == VK_DOWN) {
		xangle += rotrads;
		return 0;
	}

	break;
	case WM_MOVE:
	    if(gfx) gfx->WindowMoved();
		return 0;
	break;
	case WM_CLOSE:
   		State = STATE_QUIT;
   		CleanUp();
   		DestroyWindow(hWnd);
		return 0;
	break;
	case WM_DESTROY:
   		PostQuitMessage(0);
	break;

	case WM_MOUSEMOVE:
		MouseX = LOWORD(lParam);
		MouseY = HIWORD(lParam);
		return 0;
	break;

	default: break;
	}

	return DefWindowProc(hWnd, msg, wParam, lParam);
}

DWORD StartTicks;
Vertex vert[8];				// cube vertices
Matrix4x4 wcstm;			// world coordinate system transformation matrix
Matrix4x4 wcstrans;			// world coordinate system translation matrix
Vertex pervert[8];				// salooooooooooooooooooooooo
Triangle cube[12];
int *LeftX, *RightX;
WORD *LeftZ, *RightZ;
float FOV = quarter_pi;
float dist = (float)WINDOWX/(float)(2*tan(FOV/2));

Triangle *tris[12];
WORD *ZBuffer;


Vertex dbgv[3];
Triangle dbgt;

// let the ring bearer decide
// we'll go though the raster
// so be it

void Init() 
{

	xangle = yangle = 0;

	gfx = new NucGFX(SizeX, SizeY, BPP, NUCGFX_WINDOWED, 2);
	ZBuffer = new WORD[SizeX*SizeY];

	LeftX = new int[WINDOWY];		// edge arrays
	RightX = new int[WINDOWY];
	LeftZ = new WORD[WINDOWY];
	RightZ = new WORD[WINDOWY];

	vert[0] = Vertex(1,1,-1);			// local coordinates
	vert[1] = Vertex(1,-1,-1);
	vert[2] = Vertex(-1,-1,-1);
	vert[3] = Vertex(-1,1,-1);
	vert[4] = Vertex(1,1,1);
	vert[5] = Vertex(1,-1,1);
	vert[6] = Vertex(-1,-1,1);
	vert[7] = Vertex(-1,1,1);

	for(int i=0; i<8; i++)
		vert[i].SetColor(0, 0, 255);

	cube[0] = Triangle(&pervert[0], &pervert[1], &pervert[2]);
	cube[1] = Triangle(&pervert[0], &pervert[2], &pervert[3]);
	cube[2] = Triangle(&pervert[3], &pervert[2], &pervert[6]);
	cube[3] = Triangle(&pervert[3], &pervert[6], &pervert[7]);
	cube[4] = Triangle(&pervert[4], &pervert[7], &pervert[6]);
	cube[5] = Triangle(&pervert[4], &pervert[6], &pervert[5]);
	cube[6] = Triangle(&pervert[0], &pervert[4], &pervert[5]);
	cube[7] = Triangle(&pervert[0], &pervert[5], &pervert[1]);
	cube[8] = Triangle(&pervert[7], &pervert[4], &pervert[0]);
	cube[9] = Triangle(&pervert[7], &pervert[0], &pervert[3]);
	cube[10] = Triangle(&pervert[5], &pervert[6], &pervert[2]);
	cube[11] = Triangle(&pervert[5], &pervert[2], &pervert[1]);

	cube[0].SetColor(255,0,0);
	cube[1].SetColor(255,0,0);
	cube[2].SetColor(255,255,0);
	cube[3].SetColor(255,255,0);
	cube[4].SetColor(255,0,255);
	cube[5].SetColor(255,0,255);
	cube[6].SetColor(0,255,0);
	cube[7].SetColor(0,255,0);
	cube[8].SetColor(0,0,255);
	cube[9].SetColor(0,0,255);
	cube[10].SetColor(50,120,200);
	cube[11].SetColor(50,120,200);


	wcstrans.Translate(0, 0, 10);

	for(i=0; i<12; i++) {
		cube[i].CalculateNormal();
		tris[i] = &cube[i];
	}

	dbgv[0] = Vertex(100, 10, 10);
	dbgv[1] = Vertex(10, 100, 100);
	dbgv[2] = Vertex(200, 150, 1000);
	dbgt = Triangle(&dbgv[0], &dbgv[1], &dbgv[2]);

	StartTicks = GetTickCount();
}

void CleanUp() 
{
	delete gfx;
	delete [] LeftX;
	delete [] RightX;
	delete [] LeftZ;
	delete [] RightZ;
	delete [] ZBuffer;
}


float angle = 0;

void Main() 
{	
	unsigned int i;
	Matrix4x4 rot;

	rot.Rotate(xangle, yangle, 0);
	wcstm.reset();
	wcstm *= rot;	
	wcstm *= wcstrans;

	memcpy(pervert, vert, 8*sizeof(Vertex));
/*
	// BackFace Culling
	int front=0;
	std::vector<Triangle*> fronttri;
	Vector vec(0, 0, 1);
	for(i=0; i<12; i++) {
		
		float cangle = vec.dotproduct(tris[i]->norm);
		if((cangle) < 0) fronttri.push_back(tris[i]);
		front++;
	}
*/

	for(i=0; i<8; i++)  {
		pervert[i].pos.transform(wcstm);
		Project(&pervert[i]);
	}

	// Z Sorting
	float zval[12];
	for(i=0; i<12; i++) {
		zval[i] = (tris[i]->vert[0]->pos.z + tris[i]->vert[1]->pos.z + tris[i]->vert[2]->pos.z);// / 3.0f;
	}

	Triangle *temptri;
	float tempz;
	for(i=12; i>0; i--) {
		for(int j=0; j<i-1; j++) {
			if(zval[j] < zval[j+1]) {
				temptri = tris[j];
				tris[j] = tris[j+1];
				tris[j+1] = temptri;
				tempz = zval[j];
				zval[j] = zval[j+1];
				zval[j+1] = tempz;
			}
		}
	}

	gfx->ClearSurf(0, gfx->BackBuffer);
	ClearZBuffer();
	gfx->Lock(gfx->BackBuffer);

//	for(i=0; i<front; i++) 
//		DrawFlat(tris[i], gfx->FrameBuffer, gfx->Pitch);
    for(i=0; i<12; i++) 
		DrawFlat(tris[i], gfx->FrameBuffer, gfx->Pitch);
//	DrawFlat(&dbgt, gfx->FrameBuffer, gfx->Pitch);

	gfx->UnLock(gfx->BackBuffer);
	gfx->Flip();
/*
	char str[50];
	sprintf(str, "Z=%d", ZBuffer[MouseY*WINDOWX+MouseX]);
	HDC hdc = GetDC(hWnd);
	SetTextColor(hdc, 0xffa0a0);
	SetBkMode(hdc, TRANSPARENT);
	TextOut(hdc, MouseX+10, MouseY, str, strlen(str));
	ReleaseDC(hWnd, hdc);
*/
	angle = 0.1f*(float)(GetTickCount()-StartTicks) / 1000.0f;
}

void DrawFlat(Triangle *tr, BYTE *buffer, long pitch)
{
	int scanlines, i, j, starty, startx;
	double d01, d02, d12;
	float dx, dy, slope;
	Vertex *vert1, *vert2;

	d01 = (tr->vert[0]->pos.y-tr->vert[1]->pos.y > 0) ? (tr->vert[0]->pos.y-tr->vert[1]->pos.y) : (tr->vert[1]->pos.y-tr->vert[0]->pos.y);
	d02 = (tr->vert[0]->pos.y-tr->vert[2]->pos.y > 0) ? (tr->vert[0]->pos.y-tr->vert[2]->pos.y) : (tr->vert[2]->pos.y-tr->vert[0]->pos.y);
	d12 = (tr->vert[1]->pos.y-tr->vert[2]->pos.y > 0) ? (tr->vert[1]->pos.y-tr->vert[2]->pos.y) : (tr->vert[2]->pos.y-tr->vert[1]->pos.y);
	
	scanlines = (d01>d02) ? (int)d01 : (int)d02;			// scanlines - greater of d01, d02
	scanlines = (scanlines>d12) ? scanlines : (int)d12;

	_asm {
		mov edi, LeftX
		mov ecx, WINDOWY
		mov eax, WINDOWX-1
		rep stosd

		mov edi, RightX
		mov ecx, WINDOWY
		xor eax, eax
		rep stosd
	}	

	for(j=0; j<3; j++)  {
		vert1 = tr->vert[j];
		vert2 = tr->vert[(j+1)%3];
		startx = (int)vert1->pos.x;
		int starty;
		int beginx;

		if(vert1->pos.y>vert2->pos.y)  {
			dx = vert1->pos.x - vert2->pos.x;
			dy = vert1->pos.y - vert2->pos.y;
			starty = (int)vert2->pos.y;
			beginx = (int)vert2->pos.x;
		}
		else  {
			dx = vert2->pos.x - vert1->pos.x;
			dy = vert2->pos.y - vert1->pos.y;
			starty = (int)vert1->pos.y;
			beginx = (int)vert1->pos.x;
		}

		slope = dx/dy;

		int ylines = (int)dy;
		for(i=0; i<=ylines; i++)  {
			int calcx = (int)(slope*(float)i) + beginx;
			if(calcx < 0) calcx = 0;
			if(calcx > WINDOWX) calcx = WINDOWX;
			int y = i+starty;
			if(y >= WINDOWY) break;
			if(y >= 0) {
				if(calcx < LeftX[y])					
					LeftX[y] = calcx;
				if(calcx > RightX[y])					
					RightX[y] = calcx;
			}
		}
	}

	starty = (tr->vert[0]->pos.y<tr->vert[1]->pos.y) ? (int)tr->vert[0]->pos.y : (int)tr->vert[1]->pos.y;
	starty = (starty<tr->vert[2]->pos.y) ? starty : (int)tr->vert[2]->pos.y;

	int endy = starty + scanlines;
	
	for(j=starty; j<=endy; j++)  {
		if(j >= WINDOWY) break;
		if(j >= 0) {
			for(i=LeftX[j]; i<RightX[j]; i++) {
				gfx->PutPixel((WORD)i, (WORD)j, tr->r, tr->g, tr->b, buffer, pitch);
			}
		}
	}
}
/*
void DrawFlat(Triangle *tr, BYTE *buffer, long pitch)
{
	int scanlines, i, j, starty, startx;
	double d01, d02, d12;
	float dx, dy, slope;
	Vertex *vert1, *vert2;

	float dz, zslope;

	d01 = (tr->vert[0]->pos.y-tr->vert[1]->pos.y > 0) ? (tr->vert[0]->pos.y-tr->vert[1]->pos.y) : (tr->vert[1]->pos.y-tr->vert[0]->pos.y);
	d02 = (tr->vert[0]->pos.y-tr->vert[2]->pos.y > 0) ? (tr->vert[0]->pos.y-tr->vert[2]->pos.y) : (tr->vert[2]->pos.y-tr->vert[0]->pos.y);
	d12 = (tr->vert[1]->pos.y-tr->vert[2]->pos.y > 0) ? (tr->vert[1]->pos.y-tr->vert[2]->pos.y) : (tr->vert[2]->pos.y-tr->vert[1]->pos.y);
	
	scanlines = (d01>d02) ? (int)d01 : (int)d02;			// scanlines - greater of d01, d02
	scanlines = (scanlines>d12) ? scanlines : (int)d12;

	_asm {
		mov edi, LeftX
		mov ecx, WINDOWY
		mov eax, WINDOWX-1
		rep stosd

		mov edi, RightX
		mov ecx, WINDOWY
		xor eax, eax
		rep stosd
	}	

	for(j=0; j<3; j++)  {
		vert1 = tr->vert[j];
		vert2 = tr->vert[(j+1)%3];
		startx = (int)vert1->pos.x;
		int starty;
		int beginx;
		int startz;

		if(vert1->pos.y>vert2->pos.y)  {
			dx = vert1->pos.x - vert2->pos.x;
			dy = vert1->pos.y - vert2->pos.y;
			dz = vert1->pos.z - vert2->pos.z;
			starty = (int)vert2->pos.y;
			beginx = (int)vert2->pos.x;
			startz = (int)vert2->pos.z;
		}
		else  {
			dx = vert2->pos.x - vert1->pos.x;
			dy = vert2->pos.y - vert1->pos.y;
			dz = vert2->pos.z - vert1->pos.z;
			starty = (int)vert1->pos.y;
			beginx = (int)vert1->pos.x;
			startz = (int)vert1->pos.z;
		}

		if(dy) {
			slope = dx / dy;
			zslope = dz / dy;
		} else {
			slope = 1;
			zslope = 1;
		}

		int ylines = (int)dy;
		for(i=0; i<=ylines; i++)  {
			int calcx = (int)(slope*(float)i) + beginx;
			int calcz = (int)(zslope*(float)i) + startz;
			if(calcx < 0) calcx = 0;
			if(calcx > WINDOWX) calcx = WINDOWX;
			int y = i+starty;
			if(y >= WINDOWY) break;
			if(y >= 0) {
				if(calcx < LeftX[y]) {
					LeftX[y] = calcx;
					LeftZ[y] = (WORD)calcz;
				}
					
				if(calcx > RightX[y]) {
					RightX[y] = calcx;
					RightZ[y] = (WORD)calcz;
				}
			}
		}
	}

	starty = (tr->vert[0]->pos.y<tr->vert[1]->pos.y) ? (int)tr->vert[0]->pos.y : (int)tr->vert[1]->pos.y;
	starty = (starty<tr->vert[2]->pos.y) ? starty : (int)tr->vert[2]->pos.y;

	int endy = starty + scanlines;
	
	for(j=starty; j<=endy; j++)  {
		if(j >= WINDOWY) break;
		if(j >= 0) {
			float dx = RightX[j] - LeftX[j];
			if(!dx) dx = 1;
			float zslope = (RightZ[j] - LeftZ[j]) / dx;
			for(i=LeftX[j]; i<RightX[j]; i++) {
				WORD z = (WORD)(zslope * (float)j) + LeftZ[j];
				if(z < ZBuffer[j*SizeX+i]) {
					/*
					gfx->PutPixel((WORD)i-150, (WORD)j, tr->r, tr->g, tr->b, buffer, pitch);
					WORD zc = z>>3;
					if(zc > 255) zc = 255;
					gfx->PutPixel((WORD)i+150, (WORD)j, zc, zc, zc, buffer, pitch);
					ZBuffer[j*SizeX+i] = z;
				
					if(tr == &dbgt) {
						WORD zc = z>>4;
						if(zc > 255) zc = 255;
						gfx->PutPixel((WORD)i, (WORD)j, zc, 0, 0, buffer, pitch);
					} else {
						gfx->PutPixel((WORD)i, (WORD)j, tr->r, tr->g, tr->b, buffer, pitch);
					}
					ZBuffer[j*SizeX+i] = z;
				}
			}
		}
	}
}
*/
void Project(Vertex *vertex) 
{
    vertex->pos.x = (dist*vertex->pos.x / vertex->pos.z) + (WINDOWX >> 1);
    vertex->pos.y = (WINDOWY >> 1) - (dist*vertex->pos.y / vertex->pos.z);
	vertex->pos.z *= 100.0f;
}


void ClearZBuffer() {
	int words = (SizeX*SizeY)>>1;
	_asm {
		mov edi, ZBuffer
		mov ecx, words
		mov eax, 0xffffffff
		rep stosd
	}
}